@@ -4,51 +4,82 @@
 # This module is part of Mako and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-"""provides runtime services for templates, including Context, Namespace, and various helper functions."""
+"""provides runtime services for templates, including Context,
+Namespace, and various helper functions."""
 
 from mako import exceptions, util
 import __builtin__, inspect, sys
 
 class Context(object):
-    """provides runtime namespace, output buffer, and various callstacks for templates."""
+    """Provides runtime namespace, output buffer, and various
+    callstacks for templates.
+    
+     See :ref:`runtime_toplevel` for detail on the usage of
+    :class:`.Context`.
+    
+     """
+    
     def __init__(self, buffer, **data):
         self._buffer_stack = [buffer]
-        self._orig = data  # original data, minus the builtins
-        self._data = __builtin__.__dict__.copy() # the context data which includes builtins
+        
+        # original data, minus the builtins
+        self._orig = data
+        
+        # the context data which includes builtins
+        self._data = __builtin__.__dict__.copy()
         self._data.update(data)
         self._kwargs = data.copy()
         self._with_template = None
         self._outputting_as_unicode = None
         self.namespaces = {}
         
-        # "capture" function which proxies to the generic "capture" function
-        self._data['capture'] = lambda x, *args, **kwargs: capture(self, x, *args, **kwargs)
+        # "capture" function which proxies to the 
+        # generic "capture" function
+        self._data['capture'] = util.partial(capture, self)
         
         # "caller" stack used by def calls with content
         self.caller_stack = self._data['caller'] = CallerStack()
         
     @property
     def lookup(self):
+        """Return the :class:`.TemplateLookup` associated 
+        with this :class:`.Context`.
+        
+        """
         return self._with_template.lookup
         
     @property
     def kwargs(self):
+        """Return the dictionary of keyword argments associated with this
+        :class:`.Context`.
+        
+        """
         return self._kwargs.copy()
     
     def push_caller(self, caller):
+        """Pushes a 'caller' callable onto the callstack for
+        this :class:`.Context`."""
+        
+        
         self.caller_stack.append(caller)
         
     def pop_caller(self):
+        """Pops a 'caller' callable onto the callstack for this
+        :class:`.Context`."""
+
         del self.caller_stack[-1]
         
     def keys(self):
+        """Return a list of all names established in this :class:`.Context`."""
+
         return self._data.keys()
         
     def __getitem__(self, key):
         return self._data[key]
 
     def _push_writer(self):
-        """push a capturing buffer onto this Context and return the new Writer function."""
+        """push a capturing buffer onto this Context and return
+        the new writer function."""
         
         buf = util.FastEncodingBuffer()
         self._buffer_stack.append(buf)
@@ -74,15 +105,18 @@ class Context(object):
         return self._buffer_stack.pop()
         
     def get(self, key, default=None):
+        """Return a value from this :class:`.Context`."""
+        
         return self._data.get(key, default)
         
     def write(self, string):
-        """write a string to this Context's underlying output buffer."""
+        """Write a string to this :class:`.Context` object's
+        underlying output buffer."""
         
         self._buffer_stack[-1].write(string)
         
     def writer(self):
-        """return the current writer function"""
+        """Return the current writer function"""
 
         return self._buffer_stack[-1].write
 
@@ -97,15 +131,21 @@ class Context(object):
         c.namespaces = self.namespaces
         c.caller_stack = self.caller_stack
         return c
+        
     def locals_(self, d):
-        """create a new Context with a copy of this Context's current state, updated with the given dictionary."""
+        """create a new :class:`.Context` with a copy of this 
+        :class:`Context`'s current state, updated with the given dictionary."""
+        
         if len(d) == 0:
             return self
         c = self._copy()
         c._data.update(d)
         return c
+        
     def _clean_inheritance_tokens(self):
-        """create a new copy of this Context with tokens related to inheritance state removed."""
+        """create a new copy of this :class:`.Context`. with
+        tokens related to inheritance state removed."""
+
         c = self._copy()
         x = c._data
         x.pop('self', None)
@@ -130,7 +170,13 @@ class CallerStack(list):
         
         
 class Undefined(object):
-    """represents an undefined value in a template."""
+    """Represents an undefined value in a template.
+    
+    All template modules have a constant value 
+    ``UNDEFINED`` present which is an instance of this
+    object.
+    
+    """
     def __str__(self):
         raise NameError("Undefined")
     def __nonzero__(self):
@@ -151,8 +197,18 @@ class _NSAttr(object):
         raise AttributeError(key)    
     
 class Namespace(object):
-    """provides access to collections of rendering methods, which 
-      can be local, from other templates, or from imported modules"""
+    """Provides access to collections of rendering methods, which 
+      can be local, from other templates, or from imported modules.
+      
+      To access a particular rendering method referenced by a 
+      :class:`.Namespace`, use plain attribute access::
+      
+        ${some_namespace.foo(x, y, z)}
+        
+      :class:`.Namespace` also contains several built-in attributes 
+      described here.
+      
+      """
     
     def __init__(self, name, context, module=None, 
                             template=None, templateuri=None, 
@@ -180,14 +236,49 @@ class Namespace(object):
         else:
             self.callables = None
         if populate_self and self.template is not None:
-            (lclcallable, lclcontext) = _populate_self_namespace(context, self.template, self_ns=self)
+            lclcallable, lclcontext = \
+                        _populate_self_namespace(context, self.template, self_ns=self)
+    
+    template = None
+    """The :class:`.Template` object referenced by this
+        :class:`.Namespace`, if any.
+
+    """
+
+    context = None
+    """The :class:`.Context` object for this namespace.
+    
+    Namespaces are often created with copies of contexts that
+    contain slightly different data, particularly in inheritance
+    scenarios. Using the :class:`.Context` off of a :class:`.Namespace` one
+    can traverse an entire chain of templates that inherit from
+    one-another.
+
+    """
+    
     
     @property
     def module(self):
+        """The Python module referenced by this Namespace.
+        
+        If the namespace references a :class:`.Template`, then
+        this module is the equivalent of ``template.module``,
+        i.e. the generated module for the template.
+
+        """
         return self._module or self.template.module
     
     @property
     def filename(self):
+        """The path of the filesystem file used for this
+        Namespace's module or template.
+      
+        If this is a pure module-based
+        Namespace, this evaluates to ``module.__file__``. If a
+        template-based namespace, it evaluates to the original
+        template file location.
+        
+        """
         if self._module:
             return self._module.__file__
         else:
@@ -195,30 +286,80 @@ class Namespace(object):
     
     @property
     def uri(self):
+        """The uri for this Namespace's template.
+        
+        I.e. whatever was sent to :meth:`.TemplateLookup.get_template()`.
+        
+        This is the equivalent of :attr:`Template.uri`.
+
+        """
         return self.template.uri
 
     @property
     def attr(self):
+        """Access module level attributes by name. 
+        
+        This accessor allows templates to supply "scalar"
+        attributes which are particularly handy in inheritance
+        relationships. See the example in
+        :ref:`inheritance_toplevel`.
+
+        """
         if not hasattr(self, '_attr'):
             self._attr = _NSAttr(self)
         return self._attr
 
     def get_namespace(self, uri):
-        """return a namespace corresponding to the given template uri.
+        """Return a :class:`.Namespace` corresponding to the given uri.
         
-        if a relative uri, it is adjusted to that of the template of this namespace"""
+        If the given uri is a relative uri (i.e. it does not
+        contain ia leading slash ``/``), the uri is adjusted to
+        be relative to the uri of the namespace itself. This
+        method is therefore mostly useful off of the built-in
+        ``local`` namespace, described in :ref:`namespace_local`
+
+        In
+        most cases, a template wouldn't need this function, and
+        should instead use the ``<%namespace>`` tag to load
+        namespaces. However, since all ``<%namespace>`` tags are
+        evaulated before the body of a template ever runs,
+        this method can be used to locate namespaces using
+        expressions that were generated within the body code of
+        the template, or to conditionally use a particular
+        namespace.
+        
+        """
         key = (self, uri)
         if self.context.namespaces.has_key(key):
             return self.context.namespaces[key]
         else:
-            ns = Namespace(uri, self.context._copy(), templateuri=uri, calling_uri=self._templateuri) 
+            ns = Namespace(uri, self.context._copy(), 
+                                templateuri=uri, 
+                                calling_uri=self._templateuri) 
             self.context.namespaces[key] = ns
             return ns
     
     def get_template(self, uri):
+        """Return a :class:`.Template` from the given uri.
+        
+        The uri resolution is relative to the uri of this :class:`.Namespace`
+        object's :class:`.Template`.
+        
+        """
         return _lookup_template(self.context, uri, self._templateuri)
         
     def get_cached(self, key, **kwargs):
+        """Return a value from the :class:`.Cache` referenced by this 
+        :class:`.Namespace` object's :class:`.Template`.
+        
+        The advantage to this method versus direct access to the 
+        :class:`.Cache` is that the configuration parameters
+        declared in ``<%page>`` take effect here, thereby calling
+        up the same configured backend as that configured
+        by ``<%page>``.
+        
+        """
+        
         if self.template:
             if not self.template.cache_enabled:
                 createfunc = kwargs.get('createfunc', None)
@@ -237,10 +378,15 @@ class Namespace(object):
     
     @property
     def cache(self):
+        """Return the :class:`.Cache` object referenced by this :class:`.Namespace` object's
+        :class:`.Template`.
+        
+        """
         return self.template.cache
     
     def include_file(self, uri, **kwargs):
-        """include a file at the given uri"""
+        """Include a file at the given uri"""
+        
         _include_file(self.context, uri, self._templateuri, **kwargs)
         
     def _populate(self, d, l):
@@ -258,13 +404,13 @@ class Namespace(object):
         if self.template:
             def get(key):
                 callable_ = self.template._get_def_callable(key)
-                return lambda *args, **kwargs:callable_(self.context, *args, **kwargs)
+                return util.partial(callable_, self.context)
             for k in self.template.module._exports:
                 yield (k, get(k))
         if self._module:
             def get(key):
                 callable_ = getattr(self._module, key)
-                return lambda *args, **kwargs:callable_(self.context, *args, **kwargs)
+                return util.partial(callable_, self.context)
             for k in dir(self._module):
                 if k[0] != '_':
                     yield (k, get(k))
@@ -275,18 +421,25 @@ class Namespace(object):
 
         if self.template and self.template.has_def(key):
             callable_ = self.template._get_def_callable(key)
-            return lambda *args, **kwargs:callable_(self.context, *args, **kwargs)
+            return util.partial(callable_, self.context)
 
         if self._module and hasattr(self._module, key):
             callable_ = getattr(self._module, key)
-            return lambda *args, **kwargs:callable_(self.context, *args, **kwargs)
+            return util.partial(callable_, self.context)
 
         if self.inherits is not None:
             return getattr(self.inherits, key)
-        raise AttributeError("Namespace '%s' has no member '%s'" % (self.name, key))
+        raise AttributeError(
+                    "Namespace '%s' has no member '%s'" % 
+                    (self.name, key))
 
 def supports_caller(func):
-    """apply a caller_stack compatibility decorator to a plain Python function."""
+    """Apply a caller_stack compatibility decorator to a plain
+    Python function.
+    
+    See the example in :ref:`namespaces_python_modules`.
+    
+    """
     
     def wrap_stackframe(context,  *args, **kwargs):
         context.caller_stack._push_frame()
@@ -297,13 +450,18 @@ def supports_caller(func):
     return wrap_stackframe
         
 def capture(context, callable_, *args, **kwargs):
-    """execute the given template def, capturing the output into a buffer."""
+    """Execute the given template def, capturing the output into
+    a buffer.
+    
+    See the example in :ref:`namespaces_python_modules`.
+    
+    """
     
     if not callable(callable_):
         raise exceptions.RuntimeException(
-                                "capture() function expects a callable as "
-                                "its argument (i.e. capture(func, *args, **kwargs))"
-                            )
+                            "capture() function expects a callable as "
+                            "its argument (i.e. capture(func, *args, **kwargs))"
+                        )
     context._push_buffer()
     try:
         callable_(*args, **kwargs)
@@ -334,15 +492,20 @@ def _decorate_inline(context, fn):
     return decorate_render
             
 def _include_file(context, uri, calling_uri, **kwargs):
-    """locate the template from the given uri and include it in the current output."""
+    """locate the template from the given uri and include it in
+    the current output."""
     
     template = _lookup_template(context, uri, calling_uri)
-    (callable_, ctx) = _populate_self_namespace(context._clean_inheritance_tokens(), template)
+    (callable_, ctx) = _populate_self_namespace(
+                                context._clean_inheritance_tokens(), 
+                                template)
     callable_(ctx, **_kwargs_for_include(callable_, context._orig, **kwargs))
         
 def _inherit_from(context, uri, calling_uri):
-    """called by the _inherit method in template modules to set up the inheritance chain at the start
-    of a template's execution."""
+    """called by the _inherit method in template modules to set
+    up the inheritance chain at the start of a template's
+    execution."""
+
     if uri is None:
         return None
     template = _lookup_template(context, uri, calling_uri)
@@ -351,7 +514,10 @@ def _inherit_from(context, uri, calling_uri):
     while ih.inherits is not None:
         ih = ih.inherits
     lclcontext = context.locals_({'next':ih})
-    ih.inherits = Namespace("self:%s" % template.uri, lclcontext, template = template, populate_self=False)
+    ih.inherits = Namespace("self:%s" % template.uri, 
+                                lclcontext, 
+                                template = template, 
+                                populate_self=False)
     context._data['parent'] = lclcontext._data['local'] = ih.inherits
     callable_ = getattr(template.module, '_mako_inherit', None)
     if callable_ is not None:
@@ -367,7 +533,9 @@ def _inherit_from(context, uri, calling_uri):
 def _lookup_template(context, uri, relativeto):
     lookup = context._with_template.lookup
     if lookup is None:
-        raise exceptions.TemplateLookupException("Template '%s' has no TemplateLookup associated" % context._with_template.uri)
+        raise exceptions.TemplateLookupException(
+                            "Template '%s' has no TemplateLookup associated" % 
+                            context._with_template.uri)
     uri = lookup.adjust_uri(uri, relativeto)
     try:
         return lookup.get_template(uri)
@@ -376,7 +544,9 @@ def _lookup_template(context, uri, relativeto):
 
 def _populate_self_namespace(context, template, self_ns=None):
     if self_ns is None:
-        self_ns = Namespace('self:%s' % template.uri, context, template=template, populate_self=False)
+        self_ns = Namespace('self:%s' % template.uri, 
+                                context, template=template, 
+                                populate_self=False)
     context._data['self'] = context._data['local'] = self_ns
     if hasattr(template.module, '_mako_inherit'):
         ret = template.module._mako_inherit(template, context)
@@ -385,7 +555,8 @@ def _populate_self_namespace(context, template, self_ns=None):
     return (template.callable_, context)
 
 def _render(template, callable_, args, data, as_unicode=False):
-    """create a Context and return the string output of the given template and template callable."""
+    """create a Context and return the string 
+    output of the given template and template callable."""
 
     if as_unicode:
         buf = util.FastEncodingBuffer(unicode=True)
@@ -400,7 +571,8 @@ def _render(template, callable_, args, data, as_unicode=False):
     context._outputting_as_unicode = as_unicode
     context._with_template = template
     
-    _render_context(template, callable_, context, *args, **_kwargs_for_callable(callable_, data))
+    _render_context(template, callable_, context, *args, 
+                            **_kwargs_for_callable(callable_, data))
     return context._pop_buffer().getvalue()
 
 def _kwargs_for_callable(callable_, data):
@@ -427,7 +599,8 @@ def _kwargs_for_include(callable_, data, **kwargs):
     
 def _render_context(tmpl, callable_, context, *args, **kwargs):
     import mako.template as template
-    # create polymorphic 'self' namespace for this template with possibly updated context
+    # create polymorphic 'self' namespace for this 
+    # template with possibly updated context
     if not isinstance(tmpl, template.DefTemplate):
         # if main render method, call from the base of the inheritance stack
         (inherit, lclcontext) = _populate_self_namespace(context, tmpl)
@@ -438,13 +611,16 @@ def _render_context(tmpl, callable_, context, *args, **kwargs):
         _exec_template(callable_, context, args=args, kwargs=kwargs)
         
 def _exec_template(callable_, context, args=None, kwargs=None):
-    """execute a rendering callable given the callable, a Context, and optional explicit arguments
+    """execute a rendering callable given the callable, a
+    Context, and optional explicit arguments
 
-    the contextual Template will be located if it exists, and the error handling options specified
-    on that Template will be interpreted here.
+    the contextual Template will be located if it exists, and
+    the error handling options specified on that Template will
+    be interpreted here.
     """
     template = context._with_template
-    if template is not None and (template.format_exceptions or template.error_handler):
+    if template is not None and \
+            (template.format_exceptions or template.error_handler):
         error = None
         try:
             callable_(context, *args, **kwargs)
@@ -456,7 +632,6 @@ def _exec_template(callable_, context, args=None, kwargs=None):
     else:
         callable_(context, *args, **kwargs)
 
-
 def _render_error(template, context, error):
     if template.error_handler:
         result = template.error_handler(context, error)
